import logging
from datetime import timedelta

from flask import Blueprint, request, jsonify, make_response, redirect, session
from mxcube3.core.util import networkutils
from flask_login import current_user


def deny_access(msg):
    resp = jsonify({"msg": msg})
    resp.code = 409
    return resp


def init_route(app, server, url_prefix):
    bp = Blueprint("login", __name__, url_prefix=url_prefix)

    @bp.route("/", methods=["POST"])
    def login():
        """
        Login into mxcube application.

        :returns: Response Object, Content-Type: application/json, an object
        containing following info:

        {'status':{ 'code': 'ok', 'msg': msg },
            'Proposal': proposal,
            'session': todays_session,
            'local_contact': local_contact,
            'person': someone,
            'laboratory': a_laboratory]}

        Status code set to:
        200: On success
        409: Error, could not log in
        """
        params = request.get_json()
        login_id = params.get("proposal", "")
        password = params.get("password", "")

        try:
            res = jsonify(app.usermanager.login(login_id, password))
        except Exception as ex:
            msg = "[LOGIN] User %s could not login (%s)" % (login_id, str(ex))
            logging.getLogger("MX3.HWR").info(msg)
            res = deny_access(str(ex))

        session.permanent = True

        return res

    @bp.route("/signout")
    @server.restrict
    def signout():
        """
        Signout from Mxcube3 and reset the session
        """
        app.usermanager.signout()

        return redirect("/login", code=302)

    @bp.route("/login_info", methods=["GET"])
    def login_info():
        """
        Retrieve session/login info

        :returns: Response Object, Content-Type: application/json, an object
                containing:

        {'synchrotron_name': synchrotron_name,
        'beamline_name': beamline_name,
        'loginType': loginType,
        'loginRes': {'status':{ 'code': 'ok', 'msg': msg },
        'Proposal': proposal, 'session': todays_session,
        'local_contact': local_contact,
        'person': someone,
        'laboratory': a_laboratory']}}

        Status code set to:
        200: On success
        409: Error, could not log in
        """
        user, res = app.usermanager.login_info()

        # Redirect the user to login page if for some reason logged out
        # i.e. server restart
        if not user:
            response = redirect("/login", code=302)
        else:
            response = jsonify(res)

        return response

    @bp.route("/send_feedback", methods=["POST"])
    @server.restrict
    def send_feedback():
        sender_data = request.get_json()
        sender_data["LOGGED_IN_USER"] = current_user.nickname
        networkutils.send_feedback(sender_data)
        return make_response("", 200)

    @bp.route("/refresh_session", methods=["GET"])
    @server.restrict
    def refresh_session():
        logging.getLogger("MX3.HWR").debug("Session refresh")
        server.flask.permanent_session_lifetime = timedelta(minutes=1)
        app.usermanager.update_active_users()
        return make_response("", 200)
    return bp



import logging

from flask import Blueprint, Response

from mxcubecore import HardwareRepository as HWR

from mxcube3.routes import signals
from mxcube3.core.util.convertutils import to_camel


def init_route(app, server, url_prefix):
    bp = Blueprint("mockup", __name__, url_prefix=url_prefix)

    @bp.route("/isready", methods=["GET"])
    @server.restrict
    def mockup_ready():
        logging.getLogger("HWR").info("[Routes] Called mockup ready")
        return str(app.resolution.isReady())

    @bp.route("/newres/<int:newres>", methods=["PUT"])
    @server.restrict
    def mockup_newres(newres):
        logging.getLogger("HWR").info("[Routes] Called mockup setting new resolution")
        return app.mockups.setResolution(newres)

    @bp.route("/diff_plan/<sid>", methods=["GET"])
    @server.restrict
    def create_diff_plan(sid):
        """Juts for creating a diff plan as if it were created by edna and so on."""

        acq_parameters = HWR.beamline.get_default_acquisition_parameters()
        ftype = HWR.beamline.detector_hwobj.get_property("file_suffix")
        ftype = ftype if ftype else ".?"

        task = {
            "parameters": {
                "first_image": acq_parameters.first_image,
                "num_images": 111,
                "osc_start": acq_parameters.osc_start,
                "osc_range": 42,
                "kappa": acq_parameters.kappa,
                "kappa_phi": acq_parameters.kappa_phi,
                "overlap": acq_parameters.overlap,
                "exp_time": 456,
                "num_passes": acq_parameters.num_passes,
                "resolution": acq_parameters.resolution,
                "energy": acq_parameters.energy,
                "transmission": acq_parameters.transmission,
                "shutterless": acq_parameters.shutterless,
                "detector_mode": acq_parameters.detector_mode,
                "inverse_beam": False,
                "take_dark_current": True,
                "skip_existing_images": False,
                "take_snapshots": True,
                "helical": False,
                "mesh": False,
                "prefixTemplate": "{PREFIX}_{POSITION}",
                "subDirTemplate": "{ACRONYM}/{ACRONYM}-{NAME}",
                "prefix": "foo",
                "shape": "P1",  # -1
            },
            "checked": {True},
        }

        sample_model, sample_entry = app.queue.get_entry(sid)
        dc_model, dc_entry = app.queue._create_dc(task)
        app.queue.set_dc_params(dc_model, dc_entry, task, sample_model)
        pt = dc_model.acquisitions[0].path_template

        if HWR.beamline.queue_model.check_for_path_collisions(pt):
            msg = "[QUEUE] data collection could not be added to sample: "
            msg += "path collision"
            raise Exception(msg)

        dc_model.set_origin(3)
        dc_model.set_enabled(False)

        char, char_entry = app.queue.get_entry(3)

        char.diffraction_plan.append([dc_model])
        HWR.beamline.queue_model.emit("diff_plan_available", (char, [dc_model]))

        return Response(status=200)

    @bp.route("/shape_mock_result/<sid>", methods=["GET"])
    def shape_mock_result(sid):
        shape = HWR.beamline.sample_view.camera.get_shape(sid)
        hm = {}
        cm = {}

        if shape:
            from random import random

            for i in range(1, shape.num_rows * shape.num_cols + 1):
                hm[i] = [
                    i,
                    [
                        int(random() * 255),
                        int(random() * 255),
                        int(random() * 255),
                        int(random()),
                    ],
                ]

            for i in range(1, shape.num_rows * shape.num_cols + 1):
                cm[i] = [
                    i,
                    [
                        int(random() * 255),
                        int(random() * 255),
                        int(random() * 255),
                        int(random()),
                    ],
                ]

        res = {"heatmap": hm, "crystalmap": cm}

        HWR.beamline.sample_view.camera.set_grid_data(sid, res)
        signals.grid_result_available(to_camel(shape.as_dict()))

        return Response(status=200)

    return bp



def export(func):
    func._export = True
    func._export_name = func.__name__

    return func


def get_adapter_cls_from_hardware_object(ho):
    from mxcubecore.HardwareObjects.abstract import (
        AbstractActuator,
        AbstractDetector,
        AbstractMachineInfo,
        AbstractBeam,
        AbstractNState,
        AbstractShutter,
        AbstractEnergy,
        AbstractMotor,
    )

    from mxcubecore.HardwareObjects import MiniDiff, GenericDiffractometer
 
    # This needs to be a direct import of DataPublisher otherwise the 
    # is instance check below fails due to different "import paths" It
    # inly works because mxcubecore adds mxcubecore.HardwareObjects to
    # sys path in __init__.py
    import DataPublisher

    from mxcube3.core.adapter.actuator_adapter import ActuatorAdapter
    from mxcube3.core.adapter.motor_adapter import MotorAdapter
    from mxcube3.core.adapter.detector_adapter import DetectorAdapter
    from mxcube3.core.adapter.machine_info_adapter import MachineInfoAdapter
    from mxcube3.core.adapter.beam_adapter import BeamAdapter
    from mxcube3.core.adapter.data_publisher_adapter import DataPublisherAdapter
    from mxcube3.core.adapter.energy_adapter import EnergyAdapter
    from mxcube3.core.adapter.diffractometer_adapter import DiffractometerAdapter
    from mxcube3.core.adapter.nstate_adapter import NStateAdapter

    if isinstance(ho, AbstractNState.AbstractNState) or isinstance(
        ho, AbstractShutter.AbstractShutter
    ):
        return NStateAdapter
    elif isinstance(ho, MiniDiff.MiniDiff) or isinstance(
        ho, GenericDiffractometer.GenericDiffractometer
    ):
        return DiffractometerAdapter
    elif isinstance(ho, AbstractEnergy.AbstractEnergy):
        return EnergyAdapter
    elif isinstance(ho, AbstractDetector.AbstractDetector):
        return DetectorAdapter
    elif isinstance(ho, AbstractMachineInfo.AbstractMachineInfo):
        return MachineInfoAdapter
    elif isinstance(ho, AbstractBeam.AbstractBeam):
        return BeamAdapter
    elif isinstance(ho, DataPublisher.DataPublisher):
        return DataPublisherAdapter
    elif isinstance(ho, AbstractMotor.AbstractMotor):
        return MotorAdapter
    elif isinstance(ho, AbstractActuator.AbstractActuator):
        return ActuatorAdapter
    else:
        return None
